#!/usr/bin/perl 

# 
#  FASt Term Recognizer
# 
#   DISTRIB/bin/WordtoFamilies.pl
#   Version 2.04/04-02-01
# 
#   Copyright (C) 2004  Christian Jacquemin, LIMSI-CNRS
# 
#   Selection of morphological and semantic families from single words
# 

# STDIN: single word rules (only useful single words: those found in 
#        at least one term)
#
# Word 'domain'                         : <cat> =   N.
# Word 'document'                       : <cat> =   N.

# IN: FILEmorphfamilies = morphological families (1 family/line)
#     the first word is the root word (actually any word in the family)
#
# document\N\: document\V\-+ documentary\A\-+ary documentary\N\-+ary 
#              documentation\N\-+ation document\N\-+

# IN: FILEsemfamilies = semantic families (1 family/line)
#
#     in the "link" mode, the first word is the origin of the links:
#
# domain\N: field\N margin\N area\N scope\N
# document\N: permit\N certificate\N warrant\N grant\N
# document\V: license\V permit\V certificate\V warrant\V
#
#     in the "class" mode, the first word represents the class
#        (any word, does not need to belong to the class):
#
# domain\N\: domain\N\ demesne\N\ land\N\
# domain1\N\: domain\N\ region\N\ realm\N\
# document\N\: document\N\ papers\N\

# STDOUT: single word rules (initial words + morph. and semantic siblings)
#
# Word 'domain' : <cat> = N 
#                 <syn> = ('domain1',N)|('sphere',N)|('world',N).
# Word 'domain1' : <cat> = N.
# Word 'extract' : <cat> = N <root> = ('extract',V).
# Word 'extract' : <cat> = V.
# Word 'extraction' : <cat> = N 
#                     <syn> = ('abstraction1',N)|('origin',N) 
#                     <root> = ('extract',V).

# syntactic check
if( (@ARGV + 0) != 3 ) {
    die "Usage: WordtoFamilies.pl <morph-link-file> <type-of-sem-link> <sem-link-file>!\n";
}
$TypeOfSemanticLink = $ARGV[1];
if( $TypeOfSemanticLink eq "-links" ) {
    $classes = 0;
}
elsif( $TypeOfSemanticLink eq "-classes" ) {
    $classes = 1;
}
else {
    die "Usage: WordtoFamilies.pl <morph-link-file> (-links|-classes) <sem-link-file>!\n";
}

# constants
$Is_morph_file = 0;
$Is_sem_file = 2;
$Is_word_in_term = 1;
$Is_word_in_family = 2;

# Type $Is_morph_file: morphological families: 
#                      stem: family (stem is in family)
# Type $Is_sem_file: semantic families:
#                      base: family (base is not in family if -links)
#                      base: family (base is in family if -classes)
sub scanFamily {
    # type = Is_morph_file (0) or Is_sem_file (2)
    local( $type ) = @_;

    # open the file of families
    $NAMEFILE = @ARGV[ $type ];
    open( FILEfamilies , "<$NAMEFILE" ) 
	  ||  die "Can't open file $NAMEFILE!" ;

    # selects the families in which one of the lemmas is in a term
    while( $line = <FILEfamilies> ) {

	$lineIni = $line;
	chop ($line);
	$line = $line . " ";

	# scans the whole family to check whether one of the lemmas is
	# in the lexicon
	$OneLemmaInLexicon = 0;
	($baseMember, $famMembers) = split ":", $line;
	($baseMemberword, $baseMemberpos, $famMember) 
	    =  split /\\/, $baseMember;

	while ($famMembers =~ m/[^ ]+?\\.+?\\.*? /g) {
	    ($word, $pos, $famMember) =  split /\\/, $&;

	    if( $LemmaInWordList{ $word."#".$pos } == $Is_word_in_term ){
		$OneLemmaInLexicon = 1;
	    }
	}

	# Morphology: one of the lemmas in this family is found in the lexicon
	# Semantics and "link" mode: the base lemma is found in the lexicon
	# Semantics and "class" mode: one of the lemmas in this family is 
	#                             found in the lexicon
	if( ( ( $type == $Is_morph_file && $OneLemmaInLexicon )
	      || ( $type == $Is_sem_file && !$classes &&
		   $LemmaInWordList{ $baseMemberword."#".$baseMemberpos } 
		   == $Is_word_in_term )
	      || ( $type == $Is_sem_file && $classes &&
		   $OneLemmaInLexicon ) ) 
	    && $baseMemberword && $baseMemberpos ) {

	    # scans the whole family to memorize the links
	    ($baseMember, $famMembers) = split ":", $line;
	    $baseMemberword =~ s/\\/\\\\/g;
	    $baseMemberword =~ s/'/\\'/g;
	    $baseMemberword =~ s/ +/\-/g;
	    $baseMemberword =~ s/\(.*\)//g;
	    @famMembers = split( / / , $famMembers );

	    # semantic families of polysemous words are not retained
	    if( @famMembers <= 10 ) {
		# adds the whole family
		while ($famMembers =~ m/[^ ]+?\\.+?\\.*? /g) {
		    ($word, $pos, $famMember) =  split /\\/, $&;
		    
		    $word =~ s/'/\\'/g;
		    $word =~ s/ +/\-/g;
		    $famMember =~ s/\\/\\\\/g;
		    $famMember =~ s/'/\\'/g;
		    $famMember =~ s/ +$//;
		    
		    # Morphology: memorizes the stem of the baseMember
		    # only if it is different from the current word
		    if( $type == $Is_morph_file && $word."#".$pos ne 
			$baseMemberword."#".$baseMemberpos ) {
			$stem{ $word."#".$pos } 
			= "('".$baseMemberword."',".$baseMemberpos.")";
		    }
		    
		    # Semantics and "link" mode: memorizes the semantic link 
		    # from the baseMember to the current word
		    elsif( $type == $Is_sem_file && !$classes ) {
			if( !$semLink{ $baseMemberword."#".$baseMemberpos } ) {
			    $semLink{ $baseMemberword."#".$baseMemberpos } 
			    = "('".$word."',".$pos.")";
			}
			# avoids repetition of links
			elsif( 
			    !($semLink{ $baseMemberword."#".$baseMemberpos } 
			          =~ /\(\'$word\'\,$pos\)/ ) ){
			    $semLink{ $baseMemberword."#".$baseMemberpos } 
			    .= "|"."('".$word."',".$pos.")";
			}
		    }

		    # Semantics and "class" mode: memorizes the semantic link 
		    # from the current word to the baseMember
		    elsif( $type == $Is_sem_file && $classes ) {
			if( !$semLink{ $word."#".$pos } ) {
			    $semLink{ $word."#".$pos } 
			    = "('".$baseMemberword."',".$baseMemberpos.")";
			}
			# avoids repetition of links
			elsif( !($semLink{ $word."#".$pos } 
			      =~ /\(\'$baseMemberword\'\,$baseMemberpos\)/ ) ){
			    $semLink{ $word."#".$pos } 
			    .= "|"."('".$baseMemberword."',".$baseMemberpos.")";
			}
		    }
		    
		    # memorizes that the word must be written 
		    if( !($LemmaInWordList{ $word."#".$pos } 
			  == $Is_word_in_term ) ) {
			$LemmaInWordList{ $word."#".$pos } 
			    = $Is_word_in_family;
		    }
		    if(!($LemmaInWordList{ $baseMemberword."#".$baseMemberpos } 
			  == $Is_word_in_term ) ) {
			$LemmaInWordList{ $baseMemberword."#".$baseMemberpos } 
		        = $Is_word_in_family;
		    }
		}
		# adds the whole family
	    }
	    # semantic families of polysemous words are not retained
	}
    }
    close( FILEfamilies );
}

# reminds the term lemmas
while(<STDIN>) {
    if( /Word +\'(.*)\' .*\<cat\> *= *([A-Za-z]+)/ ){
	$word = $1;
	$pos = $2;
        $LemmaInWordList{ $word."#".$pos } = $Is_word_in_term;
    }
}

# loads the lemmas in the families of the term lemmas
&scanFamily( $Is_morph_file );
&scanFamily( $Is_sem_file );

# prints the lemmas and their morph. and sem. links
while( ( $wordpos , $value ) = each( %LemmaInWordList ) ) {
    ( $word , $pos ) = split( /\#/ , $wordpos );

    if( $word && $pos ) {
	printf  "Word '%s' : <cat> = %s" , $word , $pos;

	if( $semLink{ $word."#".$pos } ) {
	    printf  " <syn> = %s" , $semLink{ $word."#".$pos };
	    
	}
	if( $stem{ $word."#".$pos } ) {
	    printf  " <root> = %s" , $stem{ $word."#".$pos };
	}
	printf  ".\n";
    }
}


